@@ -245,7 +245,7 @@ set_nameservice_float(const char *value, void *data, set_plugin_parameter_addon
 {
   if (data != NULL) {
     sscanf(value, "%f", (float *)data);
-    OLSR_PRINTF(1, "%s float %f\n", "Got", *(float *)data);
+    OLSR_PRINTF(1, "%s float %f\n", "Got", (double)(*(float *)data));
   } else {
     OLSR_PRINTF(0, "%s float %s\n", "Ignored", value);
   }
@@ -448,6 +448,9 @@ remove_nonvalid_names_from_list(struct name_entry *my_list, int type)
   case NAME_LATLON:
     valid = is_latlon_wellformed(my_list->name);
     break;
+  default:
+	valid = false;
+	break;
   }
 
   if (!valid) {
@@ -655,7 +658,6 @@ olsr_parser(union olsr_message *m, struct interface *in_if __attribute__ ((unuse
   union olsr_ip_addr originator;
   olsr_reltime vtime;
   int size;
-  uint16_t seqno;
 
   if (!nameservice_configured) {
     name_lazy_init();
@@ -667,10 +669,8 @@ olsr_parser(union olsr_message *m, struct interface *in_if __attribute__ ((unuse
   /* Fetch the originator of the messsage */
   if (olsr_cnf->ip_version == AF_INET) {
     memcpy(&originator, &m->v4.originator, olsr_cnf->ipsize);
-    seqno = ntohs(m->v4.seqno);
   } else {
     memcpy(&originator, &m->v6.originator, olsr_cnf->ipsize);
-    seqno = ntohs(m->v6.seqno);
   }
 
   /* Fetch the message based on IP version */
@@ -749,11 +749,11 @@ encap_namemsg(struct namemsg *msg)
       OLSR_PRINTF(0, "NAME PLUGIN: cant read latlon in file %s\n", latlon_in_file);
     }
   }
-  if (0.0 != my_lat && 0.0 != my_lon) {
+  if (0.0f != my_lat && 0.0f != my_lon) {
     char s[64];
     struct name_entry e;
     memset(&e, 0, sizeof(e));
-    sprintf(s, "%f,%f,%d", my_lat, my_lon, get_isdefhna_latlon());
+    sprintf(s, "%f,%f,%d", (double)my_lat, (double)my_lon, get_isdefhna_latlon());
     e.len = strlen(s);
     e.type = NAME_LATLON;
     e.name = s;
@@ -1105,12 +1105,12 @@ write_hosts_file(void)
       entry = list2db(list_node);
 
       for (name = entry->names; name != NULL; name = name->next) {
-        struct ipaddr_str strbuf;
-        OLSR_PRINTF(6, "%s\t%s%s\t#%s\n", olsr_ip_to_string(&strbuf, &name->ip), name->name, my_suffix,
-                    olsr_ip_to_string(&strbuf, &entry->originator));
+        struct ipaddr_str strbuf1, strbuf2;
+        OLSR_PRINTF(6, "%s\t%s%s\t#%s\n", olsr_ip_to_string(&strbuf1, &name->ip), name->name, my_suffix,
+                    olsr_ip_to_string(&strbuf2, &entry->originator));
 
-        fprintf(hosts, "%s\t%s%s\t# %s\n", olsr_ip_to_string(&strbuf, &name->ip), name->name, my_suffix,
-                olsr_ip_to_string(&strbuf, &entry->originator));
+        fprintf(hosts, "%s\t%s%s\t# %s\n", olsr_ip_to_string(&strbuf1, &name->ip), name->name, my_suffix,
+                olsr_ip_to_string(&strbuf2, &entry->originator));
 
 #ifdef MID_ENTRIES
         // write mid entries
@@ -1119,16 +1119,14 @@ write_hosts_file(void)
           char mid_prefix[MID_MAXLEN];
 
           while (alias != NULL) {
-            struct ipaddr_str midbuf;
-
             // generate mid prefix
             sprintf(mid_prefix, MID_PREFIX, mid_num);
 
-            OLSR_PRINTF(6, "%s\t%s%s%s\t# %s (mid #%i)\n", olsr_ip_to_string(&midbuf, &alias->alias), mid_prefix, name->name,
-                        my_suffix, olsr_ip_to_string(&strbuf, &entry->originator), mid_num);
+            OLSR_PRINTF(6, "%s\t%s%s%s\t# %s (mid #%i)\n", olsr_ip_to_string(&strbuf1, &alias->alias), mid_prefix, name->name,
+                        my_suffix, olsr_ip_to_string(&strbuf2, &entry->originator), mid_num);
 
-            fprintf(hosts, "%s\t%s%s%s\t# %s (mid #%i)\n", olsr_ip_to_string(&midbuf, &alias->alias), mid_prefix, name->name,
-                    my_suffix, olsr_ip_to_string(&strbuf, &entry->originator), mid_num);
+            fprintf(hosts, "%s\t%s%s%s\t# %s (mid #%i)\n", olsr_ip_to_string(&strbuf1, &alias->alias), mid_prefix, name->name,
+                    my_suffix, olsr_ip_to_string(&strbuf2, &entry->originator), mid_num);
 
             alias = alias->next_alias;
             mid_num++;
@@ -1574,7 +1572,7 @@ is_latlon_wellformed(const char *latlon_line)
   int hna = -1;
   float a = 0.0, b = 0.0;
   sscanf(latlon_line, "%f,%f,%d", &a, &b, &hna);
-  return (a != 0.0 && b != 0.0 && -1 != hna);
+  return (a != 0.0f && b != 0.0f && -1 != hna);
 }
 
 /**
