@@ -35,6 +35,12 @@
  *
  */
 
+/* Adjustments made to ensure data going out is converted to network
+ * byte ordering.  Also, to ensure incoming data is converted before
+ * it is used and before checksums are calculated as well.
+ * Rusty Haddock AE5AE -- for the HSMM-MESH project.
+ */
+
 /*
  * Dynamic linked library for the olsr.org olsr daemon
  */
@@ -339,7 +345,7 @@ add_signature(uint8_t * pck, int *size)
   *size += sizeof(struct s_olsrmsg);
 
   {
-    uint8_t checksum_cache[512 + KEYLENGTH];
+    uint8_t checksum_cache[1512 + KEYLENGTH];
     /* Create packet + key cache */
     /* First the OLSR packet + signature message - digest */
     memcpy(checksum_cache, pck, *size - SIGNATURE_SIZE);
@@ -425,7 +431,8 @@ validate_packet(struct interface *olsr_if, const char *pck, int *size)
     case (SCHEME):
       goto one_checksum_SHA;    /* Ahhh... fix this */
       break;
-
+    default:
+    	break;
     }
     break;
 
@@ -438,7 +445,7 @@ validate_packet(struct interface *olsr_if, const char *pck, int *size)
 one_checksum_SHA:
 
   {
-    uint8_t checksum_cache[512 + KEYLENGTH];
+    uint8_t checksum_cache[1512 + KEYLENGTH];
     /* Create packet + key cache */
     /* First the OLSR packet + signature message - digest */
     memcpy(checksum_cache, pck, *size - SIGNATURE_SIZE);
@@ -572,7 +579,7 @@ send_challenge(struct interface *olsr_if, const union olsr_ip_addr *new_host)
   olsr_printf(3, "[ENC]Size: %lu\n", (unsigned long)sizeof(struct challengemsg));
 
   {
-    uint8_t checksum_cache[512 + KEYLENGTH];
+    uint8_t checksum_cache[1512 + KEYLENGTH];
     /* Create packet + key cache */
     /* First the OLSR packet + signature message - digest */
     memcpy(checksum_cache, &cmsg, sizeof(struct challengemsg) - SIGNATURE_SIZE);
@@ -582,7 +589,8 @@ send_challenge(struct interface *olsr_if, const union olsr_ip_addr *new_host)
     /* Create the hash */
     CHECKSUM(checksum_cache, (sizeof(struct challengemsg) - SIGNATURE_SIZE) + KEYLENGTH, cmsg.signature);
   }
-  olsr_printf(3, "[ENC]Sending timestamp request to %s challenge 0x%x\n", olsr_ip_to_string(&buf, new_host), challenge);
+  olsr_printf(3, "[ENC]Sending timestamp request to %s challenge 0x%x\n",
+	      olsr_ip_to_string(&buf, new_host), challenge);
 
   /* Add to buffer */
   net_outbuffer_push(olsr_if, &cmsg, sizeof(struct challengemsg));
@@ -637,7 +645,7 @@ parse_cres(struct interface *olsr_if, char *in_msg)
   /* Check signature */
 
   {
-    uint8_t checksum_cache[512 + KEYLENGTH];
+    uint8_t checksum_cache[1512 + KEYLENGTH];
     /* Create packet + key cache */
     /* First the OLSR packet + signature message - digest */
     memcpy(checksum_cache, msg, sizeof(struct c_respmsg) - SIGNATURE_SIZE);
@@ -666,9 +674,16 @@ parse_cres(struct interface *olsr_if, char *in_msg)
   olsr_printf(3, "[ENC]Entry-challenge 0x%x\n", entry->challenge);
 
   {
-    uint8_t checksum_cache[512 + KEYLENGTH];
+    uint8_t checksum_cache[1512 + KEYLENGTH];
+    uint32_t netorder_challenge;
+
     /* First the challenge received */
-    memcpy(checksum_cache, &entry->challenge, 4);
+    /* But we have to calculate our hash with the challenge in
+     * network order just like the remote host did!  6-Jun-2011 AE5AE */
+    netorder_challenge = htonl(entry->challenge);
+    memcpy(checksum_cache, &netorder_challenge, sizeof(uint32_t));
+/*     memcpy(checksum_cache, &entry->challenge, 4); */
+
     /* Then the local IP */
     memcpy(&checksum_cache[sizeof(uint32_t)], &msg->originator, olsr_cnf->ipsize);
 
@@ -677,7 +692,8 @@ parse_cres(struct interface *olsr_if, char *in_msg)
   }
 
   if (memcmp(msg->res_sig, sha1_hash, SIGNATURE_SIZE) != 0) {
-    olsr_printf(1, "[ENC]Error in challenge signature from %s!\n", olsr_ip_to_string(&buf, (union olsr_ip_addr *)&msg->originator));
+    olsr_printf(1, "[ENC]Error in challenge signature from %s!\n",
+		olsr_ip_to_string(&buf, (union olsr_ip_addr *)&msg->originator));
 
     return 0;
   }
@@ -688,16 +704,23 @@ parse_cres(struct interface *olsr_if, char *in_msg)
 
   entry->challenge = 0;
   entry->validated = 1;
-  entry->diff = now.tv_sec - msg->timestamp;
+
+  /* Bring timestamp to host order before arith. 2011/05/31 AE5AE */
+  entry->diff = now.tv_sec - ntohl(msg->timestamp);
 
   /* update validtime - validated entry */
   entry->valtime = GET_TIMESTAMP(TIMESTAMP_HOLD_TIME * 1000);
 
-  olsr_printf(1, "[ENC]%s registered with diff %d!\n", olsr_ip_to_string(&buf, (union olsr_ip_addr *)&msg->originator),
+  olsr_printf(1, "[ENC]%s registered with diff %d!\n",
+	      olsr_ip_to_string(&buf, (union olsr_ip_addr *)&msg->originator),
               entry->diff);
 
   /* Send response-response */
-  send_rres(olsr_if, (union olsr_ip_addr *)&msg->originator, (union olsr_ip_addr *)&msg->destination, ntohl(msg->challenge));
+  send_rres(olsr_if, (union olsr_ip_addr *)&msg->originator,
+	    (union olsr_ip_addr *)&msg->destination, msg->challenge);
+/* 	    (union olsr_ip_addr *)&msg->destination, ntohl(msg->challenge)); */
+/* Don't give send_rres() the challenge in host order, as the checksum needs to
+ * be calculated in network order.   06-Jun-2011  AE5AE */
 
   return 1;
 }
@@ -723,7 +746,7 @@ parse_rres(char *in_msg)
   /* Check signature */
 
   {
-    uint8_t checksum_cache[512 + KEYLENGTH];
+    uint8_t checksum_cache[1512 + KEYLENGTH];
     /* Create packet + key cache */
     /* First the OLSR packet + signature message - digest */
     memcpy(checksum_cache, msg, sizeof(struct r_respmsg) - SIGNATURE_SIZE);
@@ -752,9 +775,15 @@ parse_rres(char *in_msg)
   olsr_printf(3, "[ENC]Entry-challenge 0x%x\n", entry->challenge);
 
   {
-    uint8_t checksum_cache[512 + KEYLENGTH];
+    uint8_t checksum_cache[1512 + KEYLENGTH];
+    uint32_t netorder_challenge;
+
     /* First the challenge received */
-    memcpy(checksum_cache, &entry->challenge, 4);
+    /* But we have to calculate our hash with the challenge in network order!  6-Jun-2011 AE5AE */
+    netorder_challenge = htonl(entry->challenge);
+    memcpy(checksum_cache, &netorder_challenge, sizeof(uint32_t));
+/*     memcpy(checksum_cache, &entry->challenge, 4); */
+
     /* Then the local IP */
     memcpy(&checksum_cache[sizeof(uint32_t)], &msg->originator, olsr_cnf->ipsize);
 
@@ -774,7 +803,10 @@ parse_rres(char *in_msg)
 
   entry->challenge = 0;
   entry->validated = 1;
-  entry->diff = now.tv_sec - msg->timestamp;
+
+  /* Bring timestamp to host order before arith. 2011/05/31 AE5AE */
+  entry->diff = now.tv_sec - ntohl(msg->timestamp);
+
 
   /* update validtime - validated entry */
   entry->valtime = GET_TIMESTAMP(TIMESTAMP_HOLD_TIME * 1000);
@@ -832,7 +864,7 @@ parse_challenge(struct interface *olsr_if, char *in_msg)
   /* Check signature */
 
   {
-    uint8_t checksum_cache[512 + KEYLENGTH];
+    uint8_t checksum_cache[1512 + KEYLENGTH];
     /* Create packet + key cache */
     /* First the OLSR packet + signature message - digest */
     memcpy(checksum_cache, msg, sizeof(struct challengemsg) - SIGNATURE_SIZE);
@@ -857,7 +889,11 @@ parse_challenge(struct interface *olsr_if, char *in_msg)
 
   /* Build and send response */
 
-  send_cres(olsr_if, (union olsr_ip_addr *)&msg->originator, (union olsr_ip_addr *)&msg->destination, ntohl(msg->challenge), entry);
+  send_cres(olsr_if, (union olsr_ip_addr *)&msg->originator,
+	    (union olsr_ip_addr *)&msg->destination, msg->challenge, entry);
+/* 	    (union olsr_ip_addr *)&msg->destination, ntohl(msg->challenge), entry); */
+/* Don't give send_cres() the challenge in host order, as the checksum needs to
+ * be calculated with network order.   06-Jun-2011  AE5AE */
 
   return 1;
 }
@@ -893,9 +929,12 @@ send_cres(struct interface *olsr_if, union olsr_ip_addr *to, union olsr_ip_addr
   crmsg.seqno = htons(get_msg_seqno());
 
   /* set timestamp */
-  crmsg.timestamp = now.tv_sec;
+  /* but swap the byte order to the network order before sending!  2011/05/28 AE5AE */
+  crmsg.timestamp = htonl(now.tv_sec);
 #ifndef WIN32
-  olsr_printf(3, "[ENC]Timestamp %lld\n", (long long)crmsg.timestamp);
+  /* Don't print htonl()'d time, use now.tv_sec 2011/05/31 AE5AE */
+/*   olsr_printf(3, "[ENC]Timestamp %lld\n", (long long)crmsg.timestamp); */
+  olsr_printf(3, "[ENC]Timestamp %lld\n", (long long)now.tv_sec);
 #endif
 
   /* Fill subheader */
@@ -905,7 +944,7 @@ send_cres(struct interface *olsr_if, union olsr_ip_addr *to, union olsr_ip_addr
   /* Create digest of received challenge + IP */
 
   {
-    uint8_t checksum_cache[512 + KEYLENGTH];
+    uint8_t checksum_cache[1512 + KEYLENGTH];
     /* Create packet + key cache */
     /* First the challenge received */
     memcpy(checksum_cache, &chal_in, 4);
@@ -919,7 +958,7 @@ send_cres(struct interface *olsr_if, union olsr_ip_addr *to, union olsr_ip_addr
   /* Now create the digest of the message and the key */
 
   {
-    uint8_t checksum_cache[512 + KEYLENGTH];
+    uint8_t checksum_cache[1512 + KEYLENGTH];
     /* Create packet + key cache */
     /* First the OLSR packet + signature message - digest */
     memcpy(checksum_cache, &crmsg, sizeof(struct c_respmsg) - SIGNATURE_SIZE);
@@ -963,10 +1002,13 @@ send_rres(struct interface *olsr_if, union olsr_ip_addr *to, union olsr_ip_addr
   rrmsg.seqno = htons(get_msg_seqno());
 
   /* set timestamp */
-  rrmsg.timestamp = now.tv_sec;
+  /* But swap the byte order to the network order!  2011/05/28 AE5AE */
+  rrmsg.timestamp = htonl(now.tv_sec);
 
 #ifndef WIN32
-  olsr_printf(3, "[ENC]Timestamp %lld\n", (long long)rrmsg.timestamp);
+  /* olsr_printf(3, "[ENC]Timestamp %lld\n", (long long)rrmsg.timestamp); */
+  /* don't print htonl()'d time, use now. 2011/05/31 AE5AE */
+  olsr_printf(3, "[ENC]Timestamp %lld\n", (long long)now.tv_sec);
 #endif
   /* Fill subheader */
   memcpy(&rrmsg.destination, to, olsr_cnf->ipsize);
@@ -974,7 +1016,7 @@ send_rres(struct interface *olsr_if, union olsr_ip_addr *to, union olsr_ip_addr
   /* Create digest of received challenge + IP */
 
   {
-    uint8_t checksum_cache[512 + KEYLENGTH];
+    uint8_t checksum_cache[1512 + KEYLENGTH];
     /* Create packet + key cache */
     /* First the challenge received */
     memcpy(checksum_cache, &chal_in, 4);
@@ -988,7 +1030,7 @@ send_rres(struct interface *olsr_if, union olsr_ip_addr *to, union olsr_ip_addr
   /* Now create the digest of the message and the key */
 
   {
-    uint8_t checksum_cache[512 + KEYLENGTH];
+    uint8_t checksum_cache[1512 + KEYLENGTH];
     /* Create packet + key cache */
     /* First the OLSR packet + signature message - digest */
     memcpy(checksum_cache, &rrmsg, sizeof(struct r_respmsg) - SIGNATURE_SIZE);
@@ -1056,7 +1098,8 @@ timeout_timestamps(void *foo __attribute__ ((unused)))
         entry_to_delete = tmp_list;
         tmp_list = tmp_list->next;
 
-        olsr_printf(1, "[ENC]timestamp info for %s timed out.. deleting it\n", olsr_ip_to_string(&buf, &entry_to_delete->addr));
+        olsr_printf(1, "[ENC]timestamp info for %s timed out.. deleting it\n",
+		    olsr_ip_to_string(&buf, &entry_to_delete->addr));
 
         /*Delete it */
         entry_to_delete->next->prev = entry_to_delete->prev;
