@@ -6,6 +6,7 @@
  *                     includes code by Andreas Lopatic
  *                     includes code by Sven-Ola Tuecke
  *                     includes code by Lorenz Schori
+ *                     includes bugs by Markus Kittenberger
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -91,11 +92,11 @@ static int ipc_socket;
 /* IPC initialization function */
 static int plugin_ipc_init(void);
 
-static void send_info(int /*send_what*/, int /*socket*/);
+static void send_info(unsigned int /*send_what*/, int /*socket*/);
 
 static void ipc_action(int, void *, unsigned int);
 
-static void ipc_print_neigh(struct autobuf *);
+static void ipc_print_neigh(struct autobuf *, bool);
 
 static void ipc_print_link(struct autobuf *);
 
@@ -111,18 +112,24 @@ static void ipc_print_gateway(struct autobuf *);
 
 static void ipc_print_config(struct autobuf *);
 
+static void ipc_print_interface(struct autobuf *);
+
 #define TXT_IPC_BUFSIZE 256
 
-#define SIW_ALL 0
-#define SIW_NEIGH 1
-#define SIW_LINK 2
-#define SIW_ROUTE 3
-#define SIW_HNA 4
-#define SIW_MID 5
-#define SIW_TOPO 6
-#define SIW_NEIGHLINK 7
-#define SIW_GATEWAY 8
-#define SIW_CONFIG 9
+#define SIW_NEIGH 0x0001
+#define SIW_LINK 0x0002
+#define SIW_ROUTE 0x0004
+#define SIW_HNA 0x0008
+#define SIW_MID 0x0010
+#define SIW_TOPO 0x0020
+#define SIW_GATEWAY 0x0040
+#define SIW_INTERFACE 0x0080
+#define SIW_CONFIG 0x0100
+#define SIW_2HOP 0x0200
+#define SIW_VERSION 0x0400
+
+/* ALL = neigh link route hna mid topo */
+#define SIW_ALL 0x003F
 
 #define MAX_CLIENTS 3
 
@@ -242,7 +249,7 @@ ipc_action(int fd, void *data __attribute__ ((unused)), unsigned int flags __att
   char addr[INET6_ADDRSTRLEN];
   fd_set rfds;
   struct timeval tv;
-  int send_what = 0;
+  unsigned int send_what = 0;
   int ipc_connection;
 
   socklen_t addrlen = sizeof(pin);
@@ -296,39 +303,43 @@ ipc_action(int fd, void *data __attribute__ ((unused)), unsigned int flags __att
        * page the normal output is somewhat lengthy. The
        * header parsing is sufficient for standard wget.
        */
-      if (0 != strstr(requ, "/neighbours"))
-        send_what = SIW_NEIGHLINK;
-      else if (0 != strstr(requ, "/neigh"))
-        send_what = SIW_NEIGH;
-      else if (0 != strstr(requ, "/link"))
-        send_what = SIW_LINK;
-      else if (0 != strstr(requ, "/route"))
-        send_what = SIW_ROUTE;
-      else if (0 != strstr(requ, "/hna"))
-        send_what = SIW_HNA;
-      else if (0 != strstr(requ, "/mid"))
-        send_what = SIW_MID;
-      else if (0 != strstr(requ, "/topo"))
-        send_what = SIW_TOPO;
-      else if (0 != strstr(requ, "/gateway"))
-        send_what = SIW_GATEWAY;
-      else if (0 != strstr(requ, "/config"))
-        send_what = SIW_CONFIG;
+      if (0 != strstr(requ, "/neighbours")) send_what = SIW_NEIGH | SIW_LINK;
+      else {
+        /* print out every combinations of requested tabled
+         * 3++ letter abbreviations are matched */
+        if (0 != strstr(requ, "/all")) send_what = SIW_ALL;
+        else { /*already included in /all*/
+          if (0 != strstr(requ, "/nei")) send_what |= SIW_NEIGH;
+          if (0 != strstr(requ, "/lin")) send_what |= SIW_LINK;
+          if (0 != strstr(requ, "/rou")) send_what |= SIW_ROUTE;
+          if (0 != strstr(requ, "/hna")) send_what |= SIW_HNA;
+          if (0 != strstr(requ, "/mid")) send_what |= SIW_MID;
+          if (0 != strstr(requ, "/top")) send_what |= SIW_TOPO;
+        }
+        if (0 != strstr(requ, "/gat")) send_what |= SIW_GATEWAY;
+        if (0 != strstr(requ, "/con")) send_what |= SIW_CONFIG;
+        if (0 != strstr(requ, "/int")) send_what |= SIW_INTERFACE;
+        if (0 != strstr(requ, "/2ho")) send_what |= SIW_2HOP;
+        if (0 != strstr(requ, "/ver")) send_what |= SIW_VERSION;
+      }
     }
+    if ( send_what == 0 ) send_what = SIW_ALL;
   }
 
   send_info(send_what, ipc_connection);
 }
 
 static void
-ipc_print_neigh(struct autobuf *abuf)
+ipc_print_neigh(struct autobuf *abuf, bool list_2hop)
 {
   struct ipaddr_str buf1;
   struct neighbor_entry *neigh;
   struct neighbor_2_list_entry *list_2;
   int thop_cnt;
 
-  abuf_puts(abuf, "Table: Neighbors\nIP address\tSYM\tMPR\tMPRS\tWill.\t2 Hop Neighbors\n");
+  abuf_puts(abuf, "Table: Neighbors\nIP address\tSYM\tMPR\tMPRS\tWill.");
+  if (list_2hop) abuf_puts(abuf,"\n\t2hop interface adrress\n");
+  else abuf_puts(abuf, "\t2 Hop Neighbors\n");
 
   /* Neighbors */
   OLSR_FOR_ALL_NBR_ENTRIES(neigh) {
@@ -337,10 +348,12 @@ ipc_print_neigh(struct autobuf *abuf)
     thop_cnt = 0;
 
     for (list_2 = neigh->neighbor_2_list.next; list_2 != &neigh->neighbor_2_list; list_2 = list_2->next) {
-      //size += sprintf(&buf[size], "<option>%s</option>\n", olsr_ip_to_string(&buf1, &list_2->neighbor_2->neighbor_2_addr));
-      thop_cnt++;
+      if (list_2hop) abuf_appendf(abuf, "\t%s\n", olsr_ip_to_string(&buf1, &list_2->neighbor_2->neighbor_2_addr));
+      else thop_cnt++;
+    }
+    if (!list_2hop) {
+      abuf_appendf(abuf, "%d\n", thop_cnt);
     }
-    abuf_appendf(abuf, "%d\n", thop_cnt);
   }
   OLSR_FOR_ALL_NBR_ENTRIES_END(neigh);
   abuf_puts(abuf, "\n");
@@ -442,14 +455,11 @@ ipc_print_topology(struct autobuf *abuf)
 static void
 ipc_print_hna(struct autobuf *abuf)
 {
-  int size;
   struct ip_prefix_list *hna;
   struct hna_entry *tmp_hna;
   struct hna_net *tmp_net;
   struct ipaddr_str buf, mainaddrbuf;
 
-  size = 0;
-
 #ifdef ACTIVATE_VTIME_TXTINFO
   abuf_puts(abuf, "Table: HNA\nDestination\tGateway\tVTime\n");
 #else
@@ -558,7 +568,7 @@ ipc_print_gateway(struct autobuf *abuf)
   struct lqtextbuffer lqbuf;
 
   // Status IP ETX Hopcount Uplink-Speed Downlink-Speed ipv4/ipv4-nat/- ipv6/- ipv6-prefix/-
-  abuf_puts(abuf, "Table: Gateways\n   Gateway\tETX\tHopcnt\tUplink\tDownlnk\tIPv4\tIPv6\tPrefix\n");
+  abuf_puts(abuf, "Table: Gateways\nStatus\tGateway IP\tETX\tHopcnt\tUplink\tDownlnk\tIPv4\tIPv6\tPrefix\n");
   OLSR_FOR_ALL_GATEWAY_ENTRIES(gw) {
     char v4 = '-', v6 = '-';
     bool autoV4 = false, autoV6 = false;
@@ -591,7 +601,7 @@ ipc_print_gateway(struct autobuf *abuf)
       v6type = IPV6;
     }
 
-    abuf_appendf(abuf, "%c%c %s\t%s\t%d\t%u\t%u\t%s\t%s\t%s\n",
+    abuf_appendf(abuf, "%c%c\t%s\t%s\t%d\t%u\t%u\t%s\t%s\t%s\n",
         v4, v6, olsr_ip_to_string(&buf, &gw->originator),
         get_linkcost_text(tc->path_cost, true, &lqbuf), tc->hops,
         gw->uplink, gw->downlink, v4type, v6type,
@@ -606,6 +616,41 @@ ipc_print_config(struct autobuf *abuf)
   olsrd_write_cnf_autobuf(abuf, olsr_cnf);
 }
 
+static void
+ipc_print_version(struct autobuf *abuf)
+{
+abuf_appendf(abuf, "Version: %s (built on %s on %s)\n", olsrd_version, build_date, build_host);
+}
+static void
+ipc_print_interface(struct autobuf *abuf)
+{
+  const struct olsr_if *ifs;
+  abuf_puts(abuf, "Table: Interfaces\nName\tState\tMTU\tWLAN\tSrc-Adress\tMask\tDst-Adress\n");
+  for (ifs = olsr_cnf->interfaces; ifs != NULL; ifs = ifs->next) {
+    const struct interface *const rifs = ifs->interf;
+    abuf_appendf(abuf, "%s\t", ifs->name);
+    if (!rifs) {
+      abuf_puts(abuf, "DOWN\n");
+      continue;
+    }
+    abuf_appendf(abuf, "UP\t%d\t%s\t",
+               rifs->int_mtu, rifs->is_wireless ? "Yes" : "No");
+ 
+    if (olsr_cnf->ip_version == AF_INET) {
+      struct ipaddr_str addrbuf, maskbuf, bcastbuf;
+      abuf_appendf(abuf, "%s\t%s\t%s\n",
+                 ip4_to_string(&addrbuf, rifs->int_addr.sin_addr), ip4_to_string(&maskbuf, rifs->int_netmask.sin_addr),
+                 ip4_to_string(&bcastbuf, rifs->int_broadaddr.sin_addr));
+    } else {
+       struct ipaddr_str addrbuf, maskbuf;
+      abuf_appendf(abuf, "%s\t\t%s\n",
+                 ip6_to_string(&addrbuf, &rifs->int6_addr.sin6_addr), ip6_to_string(&maskbuf, &rifs->int6_multaddr.sin6_addr));
+    }
+  }
+  abuf_puts(abuf, "\n");
+}
+
+
 static void
 txtinfo_write_data(void *foo __attribute__ ((unused))) {
   fd_set set;
@@ -659,7 +704,7 @@ txtinfo_write_data(void *foo __attribute__ ((unused))) {
 }
 
 static void
-send_info(int send_what, int the_socket)
+send_info(unsigned int send_what, int the_socket)
 {
   struct autobuf abuf;
 
@@ -671,36 +716,28 @@ send_info(int send_what, int the_socket)
 
   /* Print tables to IPC socket */
 
-  /* links + Neighbors */
-  if ((send_what == SIW_ALL) || (send_what == SIW_NEIGHLINK) || (send_what == SIW_LINK))
-    ipc_print_link(&abuf);
-
-  if ((send_what == SIW_ALL) || (send_what == SIW_NEIGHLINK) || (send_what == SIW_NEIGH))
-    ipc_print_neigh(&abuf);
-
+  /* links */
+  if ((send_what & SIW_LINK) == SIW_LINK) ipc_print_link(&abuf);
+  /* neighbours */
+  if ((send_what & SIW_NEIGH) == SIW_NEIGH) ipc_print_neigh(&abuf,false);
   /* topology */
-  if ((send_what == SIW_ALL) || (send_what == SIW_TOPO))
-    ipc_print_topology(&abuf);
-
+  if ((send_what & SIW_TOPO) == SIW_TOPO) ipc_print_topology(&abuf);
   /* hna */
-  if ((send_what == SIW_ALL) || (send_what == SIW_HNA))
-    ipc_print_hna(&abuf);
-
+  if ((send_what & SIW_HNA) == SIW_HNA) ipc_print_hna(&abuf);
   /* mid */
-  if ((send_what == SIW_ALL) || (send_what == SIW_MID))
-    ipc_print_mid(&abuf);
-
+  if ((send_what & SIW_MID) == SIW_MID) ipc_print_mid(&abuf);
   /* routes */
-  if ((send_what == SIW_ALL) || (send_what == SIW_ROUTE))
-    ipc_print_routes(&abuf);
-
+  if ((send_what & SIW_ROUTE) == SIW_ROUTE) ipc_print_routes(&abuf);
   /* gateways */
-  if (send_what == SIW_GATEWAY)
-    ipc_print_gateway(&abuf);
-
+  if ((send_what & SIW_GATEWAY) == SIW_GATEWAY) ipc_print_gateway(&abuf);
   /* config */
-  if (send_what == SIW_CONFIG)
-    ipc_print_config(&abuf);
+  if ((send_what & SIW_CONFIG) == SIW_CONFIG) ipc_print_config(&abuf);
+  /* interface */
+  if ((send_what & SIW_INTERFACE) == SIW_INTERFACE) ipc_print_interface(&abuf);
+  /* 2hop neighbour list */
+  if ((send_what & SIW_2HOP) == SIW_2HOP) ipc_print_neigh(&abuf,true);
+  /* version */
+  if ((send_what & SIW_VERSION) == SIW_VERSION) ipc_print_version(&abuf);
 
   outbuffer[outbuffer_count] = olsr_malloc(abuf.len, "txt output buffer");
   outbuffer_size[outbuffer_count] = abuf.len;
