@@ -106,7 +106,7 @@ main(int argc, char *argv[])
     exit(EXIT_FAILURE);
   }
 
-  if ((cnf = olsrd_parse_cnf(argv[1])) != NULL) {
+  if ((cnf = olsrd_parse_cnf(argv[1])) == 0) {
     if ((argc > 2) && (!strcmp(argv[2], "-print"))) {
       olsrd_print_cnf(cnf);
       olsrd_write_cnf(cnf, "./out.conf");
@@ -192,19 +192,19 @@ olsrd_print_interface_cnf(struct if_config_options *cnf, struct if_config_option
 
   printf("\tIPv6 multicast           : %s%s\n", inet_ntop(AF_INET6, &cnf->ipv6_multicast.v6, ipv6_buf, sizeof(ipv6_buf)),DEFAULT_STR(ipv6_multicast.v6));
 
-  printf("\tHELLO emission/validity  : %0.2f%s/%0.2f%s\n", cnf->hello_params.emission_interval, DEFAULT_STR(hello_params.emission_interval),
-         cnf->hello_params.validity_time,DEFAULT_STR(hello_params.validity_time));
-  printf("\tTC emission/validity     : %0.2f%s/%0.2f%s\n", cnf->tc_params.emission_interval, DEFAULT_STR(tc_params.emission_interval),
-         cnf->tc_params.validity_time,DEFAULT_STR(tc_params.validity_time));
-  printf("\tMID emission/validity    : %0.2f%s/%0.2f%s\n", cnf->mid_params.emission_interval, DEFAULT_STR(mid_params.emission_interval),
-         cnf->mid_params.validity_time,DEFAULT_STR(mid_params.validity_time));
-  printf("\tHNA emission/validity    : %0.2f%s/%0.2f%s\n", cnf->hna_params.emission_interval, DEFAULT_STR(hna_params.emission_interval),
-         cnf->hna_params.validity_time,DEFAULT_STR(hna_params.validity_time));
+  printf("\tHELLO emission/validity  : %0.2f%s/%0.2f%s\n", (double)cnf->hello_params.emission_interval, DEFAULT_STR(hello_params.emission_interval),
+		  (double)cnf->hello_params.validity_time,DEFAULT_STR(hello_params.validity_time));
+  printf("\tTC emission/validity     : %0.2f%s/%0.2f%s\n", (double)cnf->tc_params.emission_interval, DEFAULT_STR(tc_params.emission_interval),
+		  (double)cnf->tc_params.validity_time,DEFAULT_STR(tc_params.validity_time));
+  printf("\tMID emission/validity    : %0.2f%s/%0.2f%s\n", (double)cnf->mid_params.emission_interval, DEFAULT_STR(mid_params.emission_interval),
+		  (double)cnf->mid_params.validity_time,DEFAULT_STR(mid_params.validity_time));
+  printf("\tHNA emission/validity    : %0.2f%s/%0.2f%s\n", (double)cnf->hna_params.emission_interval, DEFAULT_STR(hna_params.emission_interval),
+		  (double)cnf->hna_params.validity_time,DEFAULT_STR(hna_params.validity_time));
 
   for (mult = cnf->lq_mult; mult != NULL; mult = mult->next) {
     lq_mult_cnt++;
     printf("\tLinkQualityMult          : %s %0.2f %s\n", inet_ntop(olsr_cnf->ip_version, &mult->addr, ipv6_buf, sizeof(ipv6_buf)),
-      (float)(mult->value) / 65536.0, ((lq_mult_cnt > cnf->orig_lq_mult_cnt)?" (d)":""));
+      (double)(mult->value) / (double)65536.0, ((lq_mult_cnt > cnf->orig_lq_mult_cnt)?" (d)":""));
   }
 
   printf("\tAutodetect changes       : %s%s\n", cnf->autodetect_chg ? "yes" : "no",DEFAULT_STR(autodetect_chg));
@@ -261,7 +261,7 @@ static int olsrd_sanity_check_rtpolicy(struct olsrd_config *cnf) {
     }
     else if (cnf->rt_table_pri == DEF_RT_AUTO) {
       /* choose default */
-      olsr_cnf->rt_table_pri = 0;
+      olsr_cnf->rt_table_pri = DEF_RT_NONE;
       fprintf(stderr, "No policy rule for rt_table_pri\n");
     }
 
@@ -355,7 +355,7 @@ int olsrd_sanity_check_interface_cnf(struct if_config_options * io, struct olsrd
 
   /* HELLO interval */
 
-  if (io->hello_params.validity_time < 0.0) {
+  if (io->hello_params.validity_time < 0.0f) {
     if (cnf->lq_level == 0)
       io->hello_params.validity_time = NEIGHB_HOLD_TIME;
 
@@ -364,33 +364,33 @@ int olsrd_sanity_check_interface_cnf(struct if_config_options * io, struct olsrd
   }
 
   if (io->hello_params.emission_interval < cnf->pollrate || io->hello_params.emission_interval > io->hello_params.validity_time) {
-    fprintf(stderr, "Bad HELLO parameters! (em: %0.2f, vt: %0.2f) for dev %s\n", io->hello_params.emission_interval,
-            io->hello_params.validity_time, name);
+    fprintf(stderr, "Bad HELLO parameters! (em: %0.2f, vt: %0.2f) for dev %s\n", (double)io->hello_params.emission_interval,
+    		(double)io->hello_params.validity_time, name);
     return -1;
   }
 
   /* TC interval */
   if (io->tc_params.emission_interval < cnf->pollrate || io->tc_params.emission_interval > io->tc_params.validity_time) {
-    fprintf(stderr, "Bad TC parameters! (em: %0.2f, vt: %0.2f) for dev %s\n", io->tc_params.emission_interval,
-        io->tc_params.validity_time, name);
+    fprintf(stderr, "Bad TC parameters! (em: %0.2f, vt: %0.2f) for dev %s\n", (double)io->tc_params.emission_interval,
+    		(double)io->tc_params.validity_time, name);
     return -1;
   }
 
-  if (cnf->min_tc_vtime > 0.0 && (io->tc_params.validity_time / io->tc_params.emission_interval) < 128) {
+  if (cnf->min_tc_vtime > 0.0f && (io->tc_params.validity_time / io->tc_params.emission_interval) < 128.0f) {
     fprintf(stderr, "Please use a tc vtime at least 128 times the emission interval while using the min_tc_vtime hack.\n");
     return -1;
   }
   /* MID interval */
   if (io->mid_params.emission_interval < cnf->pollrate || io->mid_params.emission_interval > io->mid_params.validity_time) {
-    fprintf(stderr, "Bad MID parameters! (em: %0.2f, vt: %0.2f) for dev %s\n", io->mid_params.emission_interval,
-            io->mid_params.validity_time, name);
+    fprintf(stderr, "Bad MID parameters! (em: %0.2f, vt: %0.2f) for dev %s\n", (double)io->mid_params.emission_interval,
+    		(double)io->mid_params.validity_time, name);
     return -1;
   }
 
   /* HNA interval */
   if (io->hna_params.emission_interval < cnf->pollrate || io->hna_params.emission_interval > io->hna_params.validity_time) {
-    fprintf(stderr, "Bad HNA parameters! (em: %0.2f, vt: %0.2f) for dev %s\n", io->hna_params.emission_interval,
-            io->hna_params.validity_time, name);
+    fprintf(stderr, "Bad HNA parameters! (em: %0.2f, vt: %0.2f) for dev %s\n", (double)io->hna_params.emission_interval,
+    		(double)io->hna_params.validity_time, name);
     return -1;
   }
 
@@ -399,7 +399,7 @@ int olsrd_sanity_check_interface_cnf(struct if_config_options * io, struct olsrd
       struct ipaddr_str buf;
 
       fprintf(stderr, "Bad Linkquality multiplier ('%s' on IP %s: %0.2f)\n",
-          name, olsr_ip_to_string(&buf, &mult->addr), (float)mult->value / (float)LINK_LOSS_MULTIPLIER);
+          name, olsr_ip_to_string(&buf, &mult->addr), (double)mult->value / (double)LINK_LOSS_MULTIPLIER);
       return -1;
     }
   }
@@ -425,12 +425,18 @@ olsrd_sanity_check_cnf(struct olsrd_config *cnf)
     return -1;
   }
 
-  /* TOS */
+  /* TOS range */
   if (cnf->tos > MAX_TOS) {
     fprintf(stderr, "TOS %d is not allowed\n", cnf->tos);
     return -1;
   }
 
+  /* TOS ECN */
+  if (cnf->tos & 0x03) {
+    fprintf(stderr, "TOS %d has set ECN bits, not allowed\n", cnf->tos);
+    return -1;
+  }
+
   if (cnf->willingness_auto == false && (cnf->willingness > MAX_WILLINGNESS)) {
     fprintf(stderr, "Willingness %d is not allowed\n", cnf->willingness);
     return -1;
@@ -438,38 +444,38 @@ olsrd_sanity_check_cnf(struct olsrd_config *cnf)
 
   /* Hysteresis */
   if (cnf->use_hysteresis == true) {
-    if (cnf->hysteresis_param.scaling < MIN_HYST_PARAM || cnf->hysteresis_param.scaling > MAX_HYST_PARAM) {
-      fprintf(stderr, "Hyst scaling %0.2f is not allowed\n", cnf->hysteresis_param.scaling);
+    if (cnf->hysteresis_param.scaling < (float)MIN_HYST_PARAM || cnf->hysteresis_param.scaling > (float)MAX_HYST_PARAM) {
+      fprintf(stderr, "Hyst scaling %0.2f is not allowed\n", (double)cnf->hysteresis_param.scaling);
       return -1;
     }
 
     if (cnf->hysteresis_param.thr_high <= cnf->hysteresis_param.thr_low) {
-      fprintf(stderr, "Hyst upper(%0.2f) thr must be bigger than lower(%0.2f) threshold!\n", cnf->hysteresis_param.thr_high,
-              cnf->hysteresis_param.thr_low);
+      fprintf(stderr, "Hyst upper(%0.2f) thr must be bigger than lower(%0.2f) threshold!\n", (double)cnf->hysteresis_param.thr_high,
+    		  (double)cnf->hysteresis_param.thr_low);
       return -1;
     }
 
-    if (cnf->hysteresis_param.thr_high < MIN_HYST_PARAM || cnf->hysteresis_param.thr_high > MAX_HYST_PARAM) {
-      fprintf(stderr, "Hyst upper thr %0.2f is not allowed\n", cnf->hysteresis_param.thr_high);
+    if (cnf->hysteresis_param.thr_high < (float)MIN_HYST_PARAM || cnf->hysteresis_param.thr_high > (float)MAX_HYST_PARAM) {
+      fprintf(stderr, "Hyst upper thr %0.2f is not allowed\n", (double)cnf->hysteresis_param.thr_high);
       return -1;
     }
 
-    if (cnf->hysteresis_param.thr_low < MIN_HYST_PARAM || cnf->hysteresis_param.thr_low > MAX_HYST_PARAM) {
-      fprintf(stderr, "Hyst lower thr %0.2f is not allowed\n", cnf->hysteresis_param.thr_low);
+    if (cnf->hysteresis_param.thr_low < (float)MIN_HYST_PARAM || cnf->hysteresis_param.thr_low > (float)MAX_HYST_PARAM) {
+      fprintf(stderr, "Hyst lower thr %0.2f is not allowed\n", (double)cnf->hysteresis_param.thr_low);
       return -1;
     }
   }
 
   /* Pollrate */
-  if (cnf->pollrate < MIN_POLLRATE || cnf->pollrate > MAX_POLLRATE) {
-    fprintf(stderr, "Pollrate %0.2f is not allowed\n", cnf->pollrate);
+  if (cnf->pollrate < (float)MIN_POLLRATE || cnf->pollrate > (float)MAX_POLLRATE) {
+    fprintf(stderr, "Pollrate %0.2f is not allowed\n", (double)cnf->pollrate);
     return -1;
   }
 
   /* NIC Changes Pollrate */
 
-  if (cnf->nic_chgs_pollrate < MIN_NICCHGPOLLRT || cnf->nic_chgs_pollrate > MAX_NICCHGPOLLRT) {
-    fprintf(stderr, "NIC Changes Pollrate %0.2f is not allowed\n", cnf->nic_chgs_pollrate);
+  if (cnf->nic_chgs_pollrate < (float)MIN_NICCHGPOLLRT || cnf->nic_chgs_pollrate > (float)MAX_NICCHGPOLLRT) {
+    fprintf(stderr, "NIC Changes Pollrate %0.2f is not allowed\n", (double)cnf->nic_chgs_pollrate);
     return -1;
   }
 
@@ -504,23 +510,27 @@ olsrd_sanity_check_cnf(struct olsrd_config *cnf)
   }
 
   /* Link quality window size */
-  if (cnf->lq_level && (cnf->lq_aging < MIN_LQ_AGING || cnf->lq_aging > MAX_LQ_AGING)) {
-    fprintf(stderr, "LQ aging factor %f is not allowed\n", cnf->lq_aging);
+  if (cnf->lq_level && (cnf->lq_aging < (float)MIN_LQ_AGING || cnf->lq_aging > (float)MAX_LQ_AGING)) {
+    fprintf(stderr, "LQ aging factor %f is not allowed\n", (double)cnf->lq_aging);
     return -1;
   }
 
   /* NAT threshold value */
-  if (cnf->lq_level && (cnf->lq_nat_thresh < 0.1 || cnf->lq_nat_thresh > 1.0)) {
-    fprintf(stderr, "NAT threshold %f is not allowed\n", cnf->lq_nat_thresh);
+  if (cnf->lq_level && (cnf->lq_nat_thresh < 0.1f || cnf->lq_nat_thresh > 1.0f)) {
+    fprintf(stderr, "NAT threshold %f is not allowed\n", (double)cnf->lq_nat_thresh);
     return -1;
   }
 
 #if defined linux
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
+#if !defined LINUX_VERSION_CODE || !defined KERNEL_VERSION
+#error "Both LINUX_VERSION_CODE and KERNEL_VERSION need to be defined"
+#else
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,24)
   if (cnf->ip_version == AF_INET6 && cnf->smart_gw_active) {
-    fprintf(stderr, "Smart gateways are not supported for linux kernel 2.4 and ipv6\n");
+    fprintf(stderr, "Smart gateways are not supported for linux kernel < 2.6.24 and ipv6\n");
     return -1;
   }
+#endif
 #endif
 
   /* this rtpolicy settings are also currently only used in Linux */
@@ -535,14 +545,30 @@ olsrd_sanity_check_cnf(struct olsrd_config *cnf)
     return -1;
   }
 
-  if (cnf->min_tc_vtime < 0.0) {
+  if (cnf->min_tc_vtime < 0.0f) {
     fprintf(stderr, "Error, negative minimal tc time not allowed.\n");
     return -1;
   }
-  if (cnf->min_tc_vtime > 0.0) {
+  if (cnf->min_tc_vtime > 0.0f) {
 	  fprintf(stderr, "Warning, you are using the min_tc_vtime hack. We hope you know what you are doing... contact olsr.org otherwise.\n");
   }
 
+  if (cnf->smart_gw_period < MIN_SMARTGW_PERIOD || cnf->smart_gw_period > MAX_SMARTGW_PERIOD) {
+    fprintf(stderr, "Error, bad gateway period: %d msec (should be %d-%d)\n",
+        cnf->smart_gw_period, MIN_SMARTGW_PERIOD, MAX_SMARTGW_PERIOD);
+    return -1;
+  }
+  if (cnf->smart_gw_stablecount < MIN_SMARTGW_STABLE || cnf->smart_gw_stablecount > MAX_SMARTGW_STABLE) {
+    fprintf(stderr, "Error, bad gateway stable count: %d (should be %d-%d)\n",
+        cnf->smart_gw_stablecount, MIN_SMARTGW_STABLE, MAX_SMARTGW_STABLE);
+    return -1;
+  }
+  if (((cnf->smart_gw_thresh < MIN_SMARTGW_THRES) || (cnf->smart_gw_thresh > MAX_SMARTGW_THRES)) && (cnf->smart_gw_thresh != 0)) {
+    fprintf(stderr, "Smart gateway threshold %d is not allowed (should be %d-%d)\n", cnf->smart_gw_thresh,
+            MIN_SMARTGW_THRES, MAX_SMARTGW_THRES);
+    return -1;
+  }
+
   if (cnf->smart_gw_type >= GW_UPLINK_CNT) {
     fprintf(stderr, "Error, illegal gateway uplink type: %d\n", cnf->smart_gw_type);
     return -1;
@@ -752,20 +778,28 @@ set_default_cnf(struct olsrd_config *cnf)
 
   cnf->smart_gw_active = DEF_SMART_GW;
   cnf->smart_gw_allow_nat = DEF_GW_ALLOW_NAT;
+  cnf->smart_gw_period = DEF_GW_PERIOD;
+  cnf->smart_gw_stablecount = DEF_GW_STABLE_COUNT;
+  cnf->smart_gw_thresh = DEF_GW_THRESH;
   cnf->smart_gw_type = DEF_GW_TYPE;
   cnf->smart_gw_uplink = DEF_UPLINK_SPEED;
   cnf->smart_gw_uplink_nat = DEF_GW_UPLINK_NAT;
   cnf->smart_gw_downlink = DEF_DOWNLINK_SPEED;
 
   cnf->use_src_ip_routes = DEF_USE_SRCIP_ROUTES;
+  cnf->set_ip_forward = true;
 
-#ifdef LINUX_NETLINK_ROUTING
+#ifdef linux
   cnf->rtnl_s = 0;
 #endif
 
-#if defined __FreeBSD__ || defined __FreeBSD_kernel__ || defined __MacOSX__ || defined __NetBSD__ || defined __OpenBSD__
+#if defined __FreeBSD__ || defined __FreeBSD_kernel__ || defined __APPLE__ || defined __NetBSD__ || defined __OpenBSD__
   cnf->rts = 0;
 #endif
+
+#ifdef HTTPINFO_PUD
+  cnf->pud_position = NULL;
+#endif
 }
 
 struct if_config_options *
@@ -823,7 +857,7 @@ olsrd_print_cnf(struct olsrd_config *cnf)
   else
     printf("No interfaces    : NOT ALLOWED\n");
   printf("TOS              : 0x%02x\n", cnf->tos);
-  printf("OlsrPort          : 0x%03x\n", cnf->olsrport);
+  printf("OlsrPort          : %d\n", cnf->olsrport);
   printf("RtTable          : %u\n", cnf->rt_table);
   printf("RtTableDefault   : %u\n", cnf->rt_table_default);
   printf("RtTableTunnel    : %u\n", cnf->rt_table_tunnel);
@@ -843,9 +877,9 @@ olsrd_print_cnf(struct olsrd_config *cnf)
     ie = ie->next;
   }
 
-  printf("Pollrate         : %0.2f\n", cnf->pollrate);
+  printf("Pollrate         : %0.2f\n", (double)cnf->pollrate);
 
-  printf("NIC ChangPollrate: %0.2f\n", cnf->nic_chgs_pollrate);
+  printf("NIC ChangPollrate: %0.2f\n", (double)cnf->nic_chgs_pollrate);
 
   printf("TC redundancy    : %d\n", cnf->tc_redundancy);
 
@@ -855,11 +889,11 @@ olsrd_print_cnf(struct olsrd_config *cnf)
 
   printf("LQ fish eye      : %d\n", cnf->lq_fish);
 
-  printf("LQ aging factor  : %f\n", cnf->lq_aging);
+  printf("LQ aging factor  : %f\n", (double)cnf->lq_aging);
 
   printf("LQ algorithm name: %s\n", cnf->lq_algorithm ? cnf->lq_algorithm : "default");
 
-  printf("NAT threshold    : %f\n", cnf->lq_nat_thresh);
+  printf("NAT threshold    : %f\n", (double)cnf->lq_nat_thresh);
 
   printf("Clear screen     : %s\n", cnf->clear_screen ? "yes" : "no");
 
@@ -869,6 +903,12 @@ olsrd_print_cnf(struct olsrd_config *cnf)
 
   printf("SmGw. Allow NAT  : %s\n", cnf->smart_gw_allow_nat ? "yes" : "no");
 
+  printf("SmGw. period     : %d\n", cnf->smart_gw_period);
+
+  printf("SmGw. stablecount: %d\n", cnf->smart_gw_stablecount);
+
+  printf("SmGw. threshold  : %d%%\n", cnf->smart_gw_thresh);
+
   printf("Smart Gw. Uplink : %s\n", GW_UPLINK_TXT[cnf->smart_gw_type]);
 
   printf("SmGw. Uplink NAT : %s\n", cnf->smart_gw_uplink_nat ? "yes" : "no");
@@ -914,8 +954,8 @@ olsrd_print_cnf(struct olsrd_config *cnf)
   /* Hysteresis */
   if (cnf->use_hysteresis) {
     printf("Using hysteresis:\n");
-    printf("\tScaling      : %0.2f\n", cnf->hysteresis_param.scaling);
-    printf("\tThr high/low : %0.2f/%0.2f\n", cnf->hysteresis_param.thr_high, cnf->hysteresis_param.thr_low);
+    printf("\tScaling      : %0.2f\n", (double)cnf->hysteresis_param.scaling);
+    printf("\tThr high/low : %0.2f/%0.2f\n", (double)cnf->hysteresis_param.thr_high, (double)cnf->hysteresis_param.thr_low);
   } else {
     printf("Not using hysteresis\n");
   }
