@@ -62,14 +62,6 @@
 #include <sys/types.h>
 #include <net/if.h>
 
-/*
- * This file contains the rtnetlink version of the linux routing code.
- * You will find the old ioctl version in kernel_routes_ioctl.c
- *
- * You can deactivate this code (and activating the ioctl one)
- * by removing the -DLINUX_NETLINK_ROUTING from make/Makefile.linux
- */
-#ifdef LINUX_NETLINK_ROUTING
 /*
  * The ARM compile complains about alignment. Copied
  * from /usr/include/linux/netlink.h and adapted for ARM
@@ -121,12 +113,12 @@ static void netlink_process_link(struct nlmsghdr *h)
   struct ifinfomsg *ifi = (struct ifinfomsg *) NLMSG_DATA(h);
   struct interface *iface;
   struct olsr_if *oif;
+  char namebuffer[IF_NAMESIZE];
 
   iface = if_ifwithindex(ifi->ifi_index);
   oif = NULL;
-  if (iface == NULL && (ifi->ifi_flags & IFF_UP) != 0) {
-    char namebuffer[IF_NAMESIZE];
 
+  if (iface == NULL && (ifi->ifi_flags & (IFF_UP|IFF_RUNNING)) == (IFF_UP|IFF_RUNNING)) {
     if (if_indextoname(ifi->ifi_index, namebuffer)) {
       if ((oif = olsrif_ifwithname(namebuffer)) != NULL) {
         /* try to take interface up, will trigger ifchange */
@@ -141,8 +133,12 @@ static void netlink_process_link(struct nlmsghdr *h)
 
   if (iface == NULL && oif == NULL) {
     /* this is not an OLSR interface */
-    olsr_trigger_ifchange(ifi->ifi_index, NULL,
-        (ifi->ifi_flags & IFF_UP) == 0 ? IFCHG_IF_REMOVE : IFCHG_IF_ADD);
+    if ((ifi->ifi_flags & IFF_UP) != 0 && (ifi->ifi_flags & IFF_RUNNING) != 0) {
+      olsr_trigger_ifchange(ifi->ifi_index, NULL, IFCHG_IF_ADD);
+    }
+    else if ((ifi->ifi_flags & IFF_UP) == 0 && (ifi->ifi_flags & IFF_RUNNING) == 0){
+      olsr_trigger_ifchange(ifi->ifi_index, NULL, IFCHG_IF_REMOVE);
+    }
   }
 }
 
@@ -178,6 +174,8 @@ static void rtnetlink_read(int sock, void *data __attribute__ ((unused)), unsign
               len, ret, plen);
       return;
     }
+
+    OLSR_PRINTF(3, "Netlink message received: type 0x%x\n", nlh->nlmsg_type);
     if ((nlh->nlmsg_type == RTM_NEWLINK) || ( nlh->nlmsg_type == RTM_DELLINK)) {
       /* handle ifup/ifdown */
       netlink_process_link(nlh);
@@ -278,15 +276,9 @@ int olsr_os_policy_rule(int family, int rttable, uint32_t priority, const char *
   req.r.rtm_family = family;
   req.r.rtm_table = rttable;
 
-  /* RTN_UNSPEC would be the wildcard, but blackhole broadcast or nat roules should usually not conflict */
-  /* -> olsr only adds deletes unicast routes */
+  /* probably unneeded */
   req.r.rtm_type = RTN_UNICAST;
 
-  /* wildcard to delete routes of all protos if no simlar-delete correct proto will get set below */
-  req.r.rtm_protocol = RTPROT_UNSPEC;
-
-  req.r.rtm_scope = RT_SCOPE_UNIVERSE;
-
   olsr_netlink_addreq(&req.n, sizeof(req), RTA_PRIORITY, &priority, sizeof(priority));
 
   if (if_name != NULL) {
@@ -363,16 +355,20 @@ static int olsr_new_netlink_route(int family, int rttable, int if_index, int met
 
   memset(&req, 0, sizeof(req));
 
-  req.n.nlmsg_len = NLMSG_LENGTH(sizeof(struct rtmsg));
-  req.n.nlmsg_flags = NLM_F_REQUEST | NLM_F_ACK;
-  if (set) {
-    req.n.nlmsg_flags |= NLM_F_CREATE | NLM_F_REPLACE;
-  }
-
-  req.n.nlmsg_type = set ? RTM_NEWROUTE : RTM_DELROUTE;
+  req.r.rtm_flags = RTNH_F_ONLINK;
   req.r.rtm_family = family;
   req.r.rtm_table = rttable;
 
+  req.n.nlmsg_len = NLMSG_LENGTH(sizeof(struct rtmsg));
+  req.n.nlmsg_flags = NLM_F_REQUEST | NLM_F_ACK;
+
+  if (set) {
+    req.n.nlmsg_flags |= NLM_F_CREATE | NLM_F_REPLACE;
+    req.n.nlmsg_type = RTM_NEWROUTE;
+  } else {
+    req.n.nlmsg_type = RTM_DELROUTE;
+  }
+
   /* RTN_UNSPEC would be the wildcard, but blackhole broadcast or nat roules should usually not conflict */
   /* -> olsr only adds deletes unicast routes */
   req.r.rtm_type = RTN_UNICAST;
@@ -389,13 +385,9 @@ static int olsr_new_netlink_route(int family, int rttable, int if_index, int met
     /* as wildcard for fuzzy deletion */
     req.r.rtm_scope = RT_SCOPE_NOWHERE;
   }
-  else if (gw) {
-    /* for multihop routes */
-    req.r.rtm_scope = RT_SCOPE_UNIVERSE;
-  }
   else {
-    /* for link neighbor routes */
-    req.r.rtm_scope = RT_SCOPE_LINK;
+    /* for all our routes */
+    req.r.rtm_scope = RT_SCOPE_UNIVERSE;
   }
 
   if (set || !del_similar) {
@@ -417,23 +409,33 @@ static int olsr_new_netlink_route(int family, int rttable, int if_index, int met
     /* add gateway */
     olsr_netlink_addreq(&req.n, sizeof(req), RTA_GATEWAY, gw, family_size);
   }
+  else {
+    if ( dst->prefix_len == 32 ) {
+      /* use destination as gateway, to 'force' linux kernel to do proper source address selection */
+      olsr_netlink_addreq(&req.n, sizeof(req), RTA_GATEWAY, &dst->prefix, family_size);
+    }
+    else {
+      /*do not use onlink on such routes(no gateway, but no hostroute aswell) -  e.g. smartgateway default route over an ptp tunnel interface*/
+      req.r.rtm_flags &= (~RTNH_F_ONLINK);
+    }
+  }
 
-  /* add destination */
+   /* add destination */
   olsr_netlink_addreq(&req.n, sizeof(req), RTA_DST, &dst->prefix, family_size);
 
   err = olsr_netlink_send(&req.n);
   if (err) {
-    if (gw) {
       struct ipaddr_str buf;
-      olsr_syslog(OLSR_LOG_ERR, ". error: %s route to %s via %s dev %s (%s %d)",
+    if (gw) {
+      olsr_syslog(OLSR_LOG_ERR, ". error: %s route to %s via %s dev %s onlink (%s %d)",
           set ? "add" : "del",
           olsr_ip_prefix_to_string(dst), olsr_ip_to_string(&buf, gw),
           if_ifwithindex_name(if_index), strerror(errno), errno);
     }
     else {
-      olsr_syslog(OLSR_LOG_ERR, ". error: %s route to %s dev %s (%s %d)",
+      olsr_syslog(OLSR_LOG_ERR, ". error: %s route to %s via %s dev %s onlink (%s %d)",
           set ? "add" : "del",
-          olsr_ip_prefix_to_string(dst), if_ifwithindex_name(if_index),
+          olsr_ip_prefix_to_string(dst), olsr_ip_to_string(&buf, &dst->prefix), if_ifwithindex_name(if_index),
           strerror(errno), errno);
     }
   }
@@ -650,7 +652,6 @@ olsr_ioctl_del_route6(const struct rt_entry *rt)
   return olsr_os_process_rt_entry(AF_INET6, rt, false);
 }
 
-#endif
 /*
  * Local Variables:
  * c-basic-offset: 2
