@@ -61,7 +61,7 @@
 #include "gateway.h"
 #include "olsr_niit.h"
 
-#ifdef LINUX_NETLINK_ROUTING
+#ifdef linux
 #include <linux/types.h>
 #include <linux/rtnetlink.h>
 #include "kernel_routes.h"
@@ -80,7 +80,9 @@ bool olsr_win32_end_flag = false;
 static void olsr_shutdown(int) __attribute__ ((noreturn));
 #endif
 
-#if defined linux || __FreeBSD__ || defined __NetBSD__ || defined __OpenBSD__
+#if defined __ANDROID__
+#define DEFAULT_LOCKFILE_PREFIX "/data/local/olsrd"
+#elif defined linux || defined __FreeBSD__ || defined __NetBSD__ || defined __OpenBSD__
 #define DEFAULT_LOCKFILE_PREFIX "/var/run/olsrd"
 #elif defined WIN32
 #define DEFAULT_LOCKFILE_PREFIX "C:\\olsrd"
@@ -124,10 +126,24 @@ struct olsr_cookie_info *def_timer_ci = NULL;
  * Additionally the lock can be killed by removing the
  * locking file.
  */
-static void olsr_create_lock_file(void) {
+static int olsr_create_lock_file(bool noExitOnFail) {
 #ifdef WIN32
-  HANDLE lck = CreateEvent(NULL, TRUE, FALSE, lock_file_name);
-  if (NULL == lck || ERROR_ALREADY_EXISTS == GetLastError()) {
+    bool success;
+    HANDLE lck;
+
+    lck = CreateFile(lock_file_name,
+            GENERIC_READ | GENERIC_WRITE,
+            FILE_SHARE_READ | FILE_SHARE_WRITE,
+            NULL,
+            OPEN_ALWAYS,
+            FILE_ATTRIBUTE_NORMAL |
+            FILE_FLAG_DELETE_ON_CLOSE,
+            NULL);
+  CreateEvent(NULL, TRUE, FALSE, lock_file_name);
+  if (INVALID_HANDLE_VALUE == lck || ERROR_ALREADY_EXISTS == GetLastError()) {
+    if (noExitOnFail) {
+      return -1;
+    }
     if (NULL == lck) {
       fprintf(stderr,
           "Error, cannot create OLSR lock '%s'.\n",
@@ -141,13 +157,30 @@ static void olsr_create_lock_file(void) {
     }
     olsr_exit("", EXIT_FAILURE);
   }
+
+  success = LockFile( lck, 0, 0, 0, 0);
+
+  if (!success) {
+      CloseHandle(lck);
+      if (noExitOnFail) {
+          return -1;
+      }
+      fprintf(stderr,
+          "Error, cannot aquire OLSR lock '%s'.\n"
+          "Another OLSR instance might be running.\n",
+          lock_file_name);
+    olsr_exit("", EXIT_FAILURE);
+  }
+      
 #else
   struct flock lck;
 
   /* create file for lock */
   lock_fd = open(lock_file_name, O_WRONLY | O_CREAT, S_IRWXU);
   if (lock_fd == 0) {
-    close(lock_fd);
+    if (noExitOnFail) {
+      return -1;
+    }
     fprintf(stderr,
         "Error, cannot create OLSR lock '%s'.\n",
         lock_file_name);
@@ -163,6 +196,9 @@ static void olsr_create_lock_file(void) {
 
   if (fcntl(lock_fd, F_SETLK, &lck) == -1) {
     close(lock_fd);
+    if (noExitOnFail) {
+      return -1;
+    }
     fprintf(stderr,
         "Error, cannot aquire OLSR lock '%s'.\n"
         "Another OLSR instance might be running.\n",
@@ -170,7 +206,7 @@ static void olsr_create_lock_file(void) {
     olsr_exit("", EXIT_FAILURE);
   }
 #endif
-  return;
+  return 0;
 }
 
 /**
@@ -205,7 +241,7 @@ int main(int argc, char *argv[]) {
   bool loadedConfig = false;
   int i;
 
-#ifdef LINUX_NETLINK_ROUTING
+#ifdef linux
   struct interface *ifn;
 #endif
 
@@ -385,7 +421,7 @@ int main(int argc, char *argv[]) {
 #endif
     olsr_exit(__func__, 0);
   }
-#ifdef LINUX_NETLINK_ROUTING
+#ifdef linux
   olsr_cnf->rtnl_s = socket(PF_NETLINK, SOCK_DGRAM, NETLINK_ROUTE);
   if (olsr_cnf->rtnl_s < 0) {
     olsr_syslog(OLSR_LOG_ERR, "rtnetlink socket: %m");
@@ -402,7 +438,7 @@ int main(int argc, char *argv[]) {
   /*
    * create routing socket
    */
-#if defined __FreeBSD__ || __FreeBSD_kernel__ || defined __MacOSX__ || defined __NetBSD__ || defined __OpenBSD__
+#if defined __FreeBSD__ || defined __FreeBSD_kernel__ || defined __APPLE__ || defined __NetBSD__ || defined __OpenBSD__
   olsr_cnf->rts = socket(PF_ROUTE, SOCK_RAW, 0);
   if (olsr_cnf->rts < 0) {
     olsr_syslog(OLSR_LOG_ERR, "routing socket: %m");
@@ -410,7 +446,7 @@ int main(int argc, char *argv[]) {
   }
 #endif
 
-#ifdef LINUX_NETLINK_ROUTING
+#ifdef linux
   /* initialize gateway system */
   if (olsr_cnf->smart_gw_active) {
     if (olsr_init_gateways()) {
@@ -428,10 +464,8 @@ int main(int argc, char *argv[]) {
   set_empty_tc_timer(GET_TIMESTAMP(0));
 
   /* enable ip forwarding on host */
-  /* Disable redirects globally */
-#ifndef WIN32
+  /* Disable redirects globally (not for WIN32) */
   net_os_set_global_ifoptions();
-#endif
 
   /* Initialize parser */
   olsr_init_parser();
@@ -461,7 +495,7 @@ int main(int argc, char *argv[]) {
     } else {
       olsr_cnf->willingness = olsr_calculate_willingness();
 
-      OLSR_PRINTF(1, "Willingness set to %d - next update in %.1f secs\n", olsr_cnf->willingness, olsr_cnf->will_int);
+      OLSR_PRINTF(1, "Willingness set to %d - next update in %.1f secs\n", olsr_cnf->willingness, (double)olsr_cnf->will_int);
     }
   }
 
@@ -514,15 +548,22 @@ int main(int argc, char *argv[]) {
   /*
    * Create locking file for olsrd, will be cleared after olsrd exits
    */
-  olsr_create_lock_file();
+  for (i=5; i>=0; i--) {
+    OLSR_PRINTF(3, "Trying to get olsrd lock...\n");
+    if (!olsr_cnf->host_emul && olsr_create_lock_file(i > 0) == 0) {
+      /* lock sucessfully created */
+      break;
+    }
+    sleep (1);
+  }
 
   /* Load plugins */
   olsr_load_plugins();
 
   OLSR_PRINTF(1, "Main address: %s\n\n", olsr_ip_to_string(&buf, &olsr_cnf->main_addr));
 
-#ifdef LINUX_NETLINK_ROUTING
-  /* create policy routing priorities if necessary */
+#ifdef linux
+  /* create policy routing rules with priorities if necessary */
   if (DEF_RT_NONE != olsr_cnf->rt_table_pri) {
     olsr_os_policy_rule(olsr_cnf->ip_version,
         olsr_cnf->rt_table, olsr_cnf->rt_table_pri, NULL, true);
@@ -536,11 +577,11 @@ int main(int argc, char *argv[]) {
         olsr_cnf->rt_table_default, olsr_cnf->rt_table_default_pri, NULL, true);
   }
 
-  /* OLSR sockets */
+  /* rule to default table on all olsrd interfaces */
   if (DEF_RT_NONE != olsr_cnf->rt_table_defaultolsr_pri) {
     for (ifn = ifnet; ifn; ifn = ifn->int_next) {
-      olsr_os_policy_rule(olsr_cnf->ip_version, olsr_cnf->rt_table_default,
-          olsr_cnf->rt_table_defaultolsr_pri, ifn->int_name, true);
+      olsr_os_policy_rule(olsr_cnf->ip_version,
+          olsr_cnf->rt_table_default, olsr_cnf->rt_table_defaultolsr_pri, ifn->int_name, true);
     }
   }
 
@@ -675,7 +716,7 @@ static void olsr_shutdown(int signo __attribute__ ((unused)))
   olsr_win32_end_request = TRUE;
 
   while (!olsr_win32_end_flag)
-  Sleep(100);
+  sleep(1);
 
   OLSR_PRINTF(1, "Scheduler stopped.\n");
 #endif
@@ -704,7 +745,7 @@ static void olsr_shutdown(int signo __attribute__ ((unused)))
 
   olsr_delete_all_mid_entries();
 
-#ifdef LINUX_NETLINK_ROUTING
+#ifdef linux
   /* trigger gateway selection */
   if (olsr_cnf->smart_gw_active) {
     olsr_cleanup_gateways();
@@ -735,7 +776,7 @@ static void olsr_shutdown(int signo __attribute__ ((unused)))
     close(ifn->olsr_socket);
     close(ifn->send_socket);
 
-#ifdef LINUX_NETLINK_ROUTING
+#ifdef linux
     if (DEF_RT_NONE != olsr_cnf->rt_table_defaultolsr_pri) {
       olsr_os_policy_rule(olsr_cnf->ip_version, olsr_cnf->rt_table_default,
           olsr_cnf->rt_table_defaultolsr_pri, ifn->int_name, false);
@@ -752,7 +793,7 @@ static void olsr_shutdown(int signo __attribute__ ((unused)))
   /* ioctl socket */
   close(olsr_cnf->ioctl_s);
 
-#ifdef LINUX_NETLINK_ROUTING
+#ifdef linux
   if (DEF_RT_NONE != olsr_cnf->rt_table_pri) {
     olsr_os_policy_rule(olsr_cnf->ip_version,
         olsr_cnf->rt_table, olsr_cnf->rt_table_pri, NULL, false);
@@ -769,7 +810,7 @@ static void olsr_shutdown(int signo __attribute__ ((unused)))
   close (olsr_cnf->rt_monitor_socket);
 #endif
 
-#if defined __FreeBSD__ || defined __FreeBSD_kernel__ || defined __MacOSX__ || defined __NetBSD__ || defined __OpenBSD__
+#if defined __FreeBSD__ || defined __FreeBSD_kernel__ || defined __APPLE__ || defined __NetBSD__ || defined __OpenBSD__
   /* routing socket */
   close(olsr_cnf->rts);
 #endif
@@ -913,9 +954,9 @@ static int olsr_process_arguments(int argc, char *argv[],
 
       sscanf(*argv, "%f", &tmp_lq_aging);
 
-      if (tmp_lq_aging < MIN_LQ_AGING || tmp_lq_aging > MAX_LQ_AGING) {
-        printf("LQ aging factor %f not allowed. Range [%f-%f]\n", tmp_lq_aging,
-            MIN_LQ_AGING, MAX_LQ_AGING);
+      if (tmp_lq_aging < (float)MIN_LQ_AGING || tmp_lq_aging > (float)MAX_LQ_AGING) {
+        printf("LQ aging factor %f not allowed. Range [%f-%f]\n", (double)tmp_lq_aging,
+        		(double)MIN_LQ_AGING, (double)MAX_LQ_AGING);
         olsr_exit(__func__, EXIT_FAILURE);
       }
       olsr_cnf->lq_aging = tmp_lq_aging;
@@ -932,9 +973,9 @@ static int olsr_process_arguments(int argc, char *argv[],
 
       sscanf(*argv, "%f", &tmp_lq_nat_thresh);
 
-      if (tmp_lq_nat_thresh < 0.1 || tmp_lq_nat_thresh > 1.0) {
+      if (tmp_lq_nat_thresh < 0.1f || tmp_lq_nat_thresh > 1.0f) {
         printf("NAT threshold %f not allowed. Range [%f-%f]\n",
-            tmp_lq_nat_thresh, 0.1, 1.0);
+        		(double)tmp_lq_nat_thresh, (double)0.1, (double)1.0);
         olsr_exit(__func__, EXIT_FAILURE);
       }
       olsr_cnf->lq_nat_thresh = tmp_lq_nat_thresh;
